@@ -17,12 +17,13 @@
 
 import os
 from unittest import mock
+import yaml
 
+import fixtures
 from oslo_config import cfg
 from oslo_context import context
 from oslo_serialization import jsonutils
 from oslotest import base as test_base
-import six
 
 from oslo_policy import _cache_handler
 from oslo_policy import _checks
@@ -296,6 +297,48 @@ class EnforcerTest(base.PolicyBaseTestCase):
             os.path.join('policy.d', 'b.conf'),
         ])
 
+    def test_load_directory_after_file_update(self):
+        self.create_config_file(
+            os.path.join('policy.d', 'a.conf'), POLICY_A_CONTENTS)
+        self.enforcer.load_rules(True)
+        self.assertIsNotNone(self.enforcer.rules)
+        loaded_rules = jsonutils.loads(str(self.enforcer.rules))
+        self.assertEqual('role:fakeA', loaded_rules['default'])
+        self.assertEqual('is_admin:True', loaded_rules['admin'])
+        self.check_loaded_files([
+            'policy.json',
+            os.path.join('policy.d', 'a.conf'),
+        ])
+        new_policy_json_contents = jsonutils.dumps({
+            "default": "rule:admin",
+            "admin": "is_admin:True",
+            "foo": "rule:bar",
+        })
+        # Modify the policy.json file and then validate that the rules
+        # from the policy directory are re-applied on top of the
+        # new rules from the file.
+        self.create_config_file('policy.json', new_policy_json_contents)
+        policy_file_path = self.get_config_file_fullname('policy.json')
+        # Force the mtime change since the unit test may write to this file
+        # too fast for mtime to actually change.
+        stinfo = os.stat(policy_file_path)
+        os.utime(policy_file_path, (stinfo.st_atime + 42,
+                                    stinfo.st_mtime + 42))
+
+        self.enforcer.load_rules()
+
+        self.assertIsNotNone(self.enforcer.rules)
+        loaded_rules = jsonutils.loads(str(self.enforcer.rules))
+        self.assertEqual('role:fakeA', loaded_rules['default'])
+        self.assertEqual('is_admin:True', loaded_rules['admin'])
+        self.assertEqual('rule:bar', loaded_rules['foo'])
+        self.check_loaded_files([
+            'policy.json',
+            os.path.join('policy.d', 'a.conf'),
+            'policy.json',
+            os.path.join('policy.d', 'a.conf'),
+        ])
+
     def test_load_directory_opts_registered(self):
         self._test_scenario_with_opts_registered(self.test_load_directory)
 
@@ -306,8 +349,7 @@ class EnforcerTest(base.PolicyBaseTestCase):
         self.enforcer.load_rules(False)
         self.assertIsNotNone(self.enforcer.rules)
 
-        old = six.next(six.itervalues(
-            self.enforcer._policy_dir_mtimes))
+        old = next(iter(self.enforcer._policy_dir_mtimes))
         self.assertEqual(1, len(self.enforcer._policy_dir_mtimes))
 
         # Touch the file
@@ -318,8 +360,7 @@ class EnforcerTest(base.PolicyBaseTestCase):
 
         self.enforcer.load_rules(False)
         self.assertEqual(1, len(self.enforcer._policy_dir_mtimes))
-        self.assertEqual(old, six.next(six.itervalues(
-            self.enforcer._policy_dir_mtimes)))
+        self.assertEqual(old, next(iter(self.enforcer._policy_dir_mtimes)))
 
         loaded_rules = jsonutils.loads(str(self.enforcer.rules))
         self.assertEqual('is_admin:True', loaded_rules['admin'])
@@ -342,14 +383,12 @@ class EnforcerTest(base.PolicyBaseTestCase):
         self.enforcer.load_rules(False)
         self.assertIsNotNone(self.enforcer.rules)
 
-        old = six.next(six.itervalues(
-            self.enforcer._policy_dir_mtimes))
+        old = next(iter(self.enforcer._policy_dir_mtimes))
         self.assertEqual(1, len(self.enforcer._policy_dir_mtimes))
 
         self.enforcer.load_rules(False)
         self.assertEqual(1, len(self.enforcer._policy_dir_mtimes))
-        self.assertEqual(old, six.next(six.itervalues(
-            self.enforcer._policy_dir_mtimes)))
+        self.assertEqual(old, next(iter(self.enforcer._policy_dir_mtimes)))
 
         loaded_rules = jsonutils.loads(str(self.enforcer.rules))
         self.assertEqual('is_admin:True', loaded_rules['admin'])
@@ -371,6 +410,34 @@ class EnforcerTest(base.PolicyBaseTestCase):
             'test_load_directory_caching_with_files_same_but_overwrite_false')  # NOQA
         self._test_scenario_with_opts_registered(test)
 
+    @mock.patch.object(policy, 'LOG')
+    def test_load_json_file_log_warning(self, mock_log):
+        rules = jsonutils.dumps({'foo': 'rule:bar'})
+        self.create_config_file('policy.json', rules)
+        self.enforcer.load_rules(True)
+
+        mock_log.warning.assert_any_call(policy.WARN_JSON)
+
+    @mock.patch.object(policy, 'LOG')
+    def test_warning_on_redundant_file_rules(self, mock_log):
+        rules = yaml.dump({'admin': 'is_admin:True'})
+        self.create_config_file('policy.yaml', rules)
+        path = self.get_config_file_fullname('policy.yaml')
+        enforcer = policy.Enforcer(self.conf, policy_file=path)
+        # register same rule in default as present in file.
+        enforcer.register_default(policy.RuleDefault(name='admin',
+                                  check_str='is_admin:True'))
+
+        enforcer.load_rules(True)
+        warn_msg = ("Policy Rules %(names)s specified in policy files "
+                    "are the same as the defaults provided by the service. "
+                    "You can remove these rules from policy files which "
+                    "will make maintenance easier. You can detect these "
+                    "redundant rules by ``oslopolicy-list-redundant`` tool "
+                    "also.")
+
+        mock_log.warning.assert_any_call(warn_msg, {'names': ['admin']})
+
     def test_load_multiple_directories(self):
         self.create_config_file(
             os.path.join('policy.d', 'a.conf'), POLICY_A_CONTENTS)
@@ -421,6 +488,7 @@ class EnforcerTest(base.PolicyBaseTestCase):
                                [os.path.join('policy.d', 'a.conf')],
                                group='oslo_policy')
         self.assertRaises(ValueError, self.enforcer.load_rules, True)
+        self.assertRaises(ValueError, self.enforcer.load_rules, False)
 
     @mock.patch('oslo_policy.policy.Enforcer.check_rules')
     def test_load_rules_twice(self, mock_check_rules):
@@ -716,6 +784,22 @@ class EnforcerTest(base.PolicyBaseTestCase):
                           policy.RuleDefault(name='owner',
                                              check_str='role:owner'))
 
+    def test_enforcer_does_not_modify_original_registered_rule(self):
+        rule_original = policy.RuleDefault(
+            name='test',
+            check_str='role:owner',)
+
+        self.enforcer.register_default(rule_original)
+        self.enforcer.registered_rules['test']._check_str = 'role:admin'
+        self.enforcer.registered_rules['test']._check = 'role:admin'
+
+        self.assertEqual(
+            self.enforcer.registered_rules['test'].check_str, 'role:admin')
+        self.assertEqual(
+            self.enforcer.registered_rules['test'].check, 'role:admin')
+        self.assertEqual(rule_original.check_str, 'role:owner')
+        self.assertEqual(rule_original.check.__str__(), 'role:owner')
+
     def test_non_reversible_check(self):
         self.create_config_file('policy.json',
                                 jsonutils.dumps(
@@ -1166,7 +1250,9 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
     def test_deprecate_a_policy_check_string(self):
         deprecated_rule = policy.DeprecatedRule(
             name='foo:create_bar',
-            check_str='role:fizz'
+            check_str='role:fizz',
+            deprecated_reason='"role:bang" is a better default',
+            deprecated_since='N'
         )
 
         rule_list = [policy.DocumentedRuleDefault(
@@ -1175,8 +1261,6 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             description='Create a bar.',
             operations=[{'path': '/v1/bars', 'method': 'POST'}],
             deprecated_rule=deprecated_rule,
-            deprecated_reason='"role:bang" is a better default',
-            deprecated_since='N'
         )]
         enforcer = policy.Enforcer(self.conf)
         enforcer.register_defaults(rule_list)
@@ -1206,7 +1290,9 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
     def test_deprecate_an_empty_policy_check_string(self):
         deprecated_rule = policy.DeprecatedRule(
             name='foo:create_bar',
-            check_str=''
+            check_str='',
+            deprecated_reason='because of reasons',
+            deprecated_since='N',
         )
 
         rule_list = [policy.DocumentedRuleDefault(
@@ -1215,8 +1301,6 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             description='Create a bar.',
             operations=[{'path': '/v1/bars', 'method': 'POST'}],
             deprecated_rule=deprecated_rule,
-            deprecated_reason='because of reasons',
-            deprecated_since='N'
         )]
         enforcer = policy.Enforcer(self.conf)
         enforcer.register_defaults(rule_list)
@@ -1234,7 +1318,9 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
     def test_deprecate_replace_with_empty_policy_check_string(self):
         deprecated_rule = policy.DeprecatedRule(
             name='foo:create_bar',
-            check_str='role:fizz'
+            check_str='role:fizz',
+            deprecated_reason='because of reasons',
+            deprecated_since='N',
         )
 
         rule_list = [policy.DocumentedRuleDefault(
@@ -1243,8 +1329,6 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             description='Create a bar.',
             operations=[{'path': '/v1/bars', 'method': 'POST'}],
             deprecated_rule=deprecated_rule,
-            deprecated_reason='because of reasons',
-            deprecated_since='N'
         )]
         enforcer = policy.Enforcer(self.conf)
         enforcer.register_defaults(rule_list)
@@ -1261,15 +1345,7 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
     def test_deprecate_a_policy_name(self):
         deprecated_rule = policy.DeprecatedRule(
             name='foo:bar',
-            check_str='role:baz'
-        )
-
-        rule_list = [policy.DocumentedRuleDefault(
-            name='foo:create_bar',
             check_str='role:baz',
-            description='Create a bar.',
-            operations=[{'path': '/v1/bars/', 'method': 'POST'}],
-            deprecated_rule=deprecated_rule,
             deprecated_reason=(
                 '"foo:bar" is not granular enough. If your deployment has '
                 'overridden "foo:bar", ensure you override the new policies '
@@ -1278,7 +1354,15 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
                 '"foo:bar:update", "foo:bar:list", and "foo:bar:delete", '
                 'which might be backwards incompatible for your deployment'
             ),
-            deprecated_since='N'
+            deprecated_since='N',
+        )
+
+        rule_list = [policy.DocumentedRuleDefault(
+            name='foo:create_bar',
+            check_str='role:baz',
+            description='Create a bar.',
+            operations=[{'path': '/v1/bars/', 'method': 'POST'}],
+            deprecated_rule=deprecated_rule,
         )]
         expected_msg = (
             'Policy "foo:bar":"role:baz" was deprecated in N in favor of '
@@ -1352,7 +1436,9 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
     def test_deprecate_check_str_suppress_does_not_log_warning(self):
         deprecated_rule = policy.DeprecatedRule(
             name='foo:create_bar',
-            check_str='role:fizz'
+            check_str='role:fizz',
+            deprecated_reason='"role:bang" is a better default',
+            deprecated_since='N'
         )
 
         rule_list = [policy.DocumentedRuleDefault(
@@ -1361,8 +1447,6 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             description='Create a bar.',
             operations=[{'path': '/v1/bars', 'method': 'POST'}],
             deprecated_rule=deprecated_rule,
-            deprecated_reason='"role:bang" is a better default',
-            deprecated_since='N'
         )]
         enforcer = policy.Enforcer(self.conf)
         enforcer.suppress_deprecation_warnings = True
@@ -1374,7 +1458,9 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
     def test_deprecate_name_suppress_does_not_log_warning(self):
         deprecated_rule = policy.DeprecatedRule(
             name='foo:bar',
-            check_str='role:baz'
+            check_str='role:baz',
+            deprecated_reason='"foo:bar" is not granular enough.',
+            deprecated_since='N',
         )
 
         rule_list = [policy.DocumentedRuleDefault(
@@ -1383,8 +1469,6 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             description='Create a bar.',
             operations=[{'path': '/v1/bars/', 'method': 'POST'}],
             deprecated_rule=deprecated_rule,
-            deprecated_reason='"foo:bar" is not granular enough.',
-            deprecated_since='N'
         )]
 
         rules = jsonutils.dumps({'foo:bar': 'role:bang'})
@@ -1422,7 +1506,9 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
     def test_suppress_default_change_warnings_flag_not_log_warning(self):
         deprecated_rule = policy.DeprecatedRule(
             name='foo:create_bar',
-            check_str='role:fizz'
+            check_str='role:fizz',
+            deprecated_reason='"role:bang" is a better default',
+            deprecated_since='N',
         )
 
         rule_list = [policy.DocumentedRuleDefault(
@@ -1431,8 +1517,6 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             description='Create a bar.',
             operations=[{'path': '/v1/bars', 'method': 'POST'}],
             deprecated_rule=deprecated_rule,
-            deprecated_reason='"role:bang" is a better default',
-            deprecated_since='N'
         )]
         enforcer = policy.Enforcer(self.conf)
         enforcer.suppress_default_change_warnings = True
@@ -1441,7 +1525,7 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             enforcer.load_rules()
             mock_warn.assert_not_called()
 
-    def test_deprecated_policy_for_removal_must_include_deprecated_since(self):
+    def test_deprecated_policy_for_removal_must_include_deprecated_meta(self):
         self.assertRaises(
             ValueError,
             policy.DocumentedRuleDefault,
@@ -1451,24 +1535,25 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             operations=[{'path': '/v1/foos/', 'method': 'POST'}],
             deprecated_for_removal=True,
             deprecated_reason='Some reason.'
+            # no deprecated_since
         )
 
-    def test_deprecated_policy_must_include_deprecated_since(self):
+    def test_deprecated_policy_should_not_include_deprecated_meta(self):
         deprecated_rule = policy.DeprecatedRule(
             name='foo:bar',
             check_str='rule:baz'
         )
 
-        self.assertRaises(
-            ValueError,
-            policy.DocumentedRuleDefault,
-            name='foo:bar',
-            check_str='rule:baz',
-            description='Create a foo.',
-            operations=[{'path': '/v1/foos/', 'method': 'POST'}],
-            deprecated_rule=deprecated_rule,
-            deprecated_reason='Some reason.'
-        )
+        with mock.patch('warnings.warn') as mock_warn:
+            policy.DocumentedRuleDefault(
+                name='foo:bar',
+                check_str='rule:baz',
+                description='Create a foo.',
+                operations=[{'path': '/v1/foos/', 'method': 'POST'}],
+                deprecated_rule=deprecated_rule,
+                deprecated_reason='Some reason.'
+            )
+            mock_warn.assert_called_once()
 
     def test_deprecated_rule_requires_deprecated_rule_object(self):
         self.assertRaises(
@@ -1504,7 +1589,9 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
         # better.
         deprecated_rule = policy.DeprecatedRule(
             name='foo:bar',
-            check_str='role:fizz'
+            check_str='role:fizz',
+            deprecated_reason='"role:bang" is a better default',
+            deprecated_since='N',
         )
         rule_list = [policy.DocumentedRuleDefault(
             name='foo:create_bar',
@@ -1512,8 +1599,6 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             description='Create a bar.',
             operations=[{'path': '/v1/bars', 'method': 'POST'}],
             deprecated_rule=deprecated_rule,
-            deprecated_reason='"role:bang" is a better default',
-            deprecated_since='N'
         )]
         self.enforcer.register_defaults(rule_list)
 
@@ -1538,7 +1623,9 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
         # better.
         deprecated_rule = policy.DeprecatedRule(
             name='foo:bar',
-            check_str='role:fizz'
+            check_str='role:fizz',
+            deprecated_reason='"role:bang" is a better default',
+            deprecated_since='N',
         )
         rule_list = [policy.DocumentedRuleDefault(
             name='foo:create_bar',
@@ -1546,8 +1633,6 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             description='Create a bar.',
             operations=[{'path': '/v1/bars', 'method': 'POST'}],
             deprecated_rule=deprecated_rule,
-            deprecated_reason='"role:bang" is a better default',
-            deprecated_since='N'
         )]
         self.enforcer.register_defaults(rule_list)
 
@@ -1580,7 +1665,9 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
         # better.
         deprecated_rule = policy.DeprecatedRule(
             name='foo:bar',
-            check_str='role:fizz'
+            check_str='role:fizz',
+            deprecated_reason='"role:bang" is a better default',
+            deprecated_since='N',
         )
         rule_list = [policy.DocumentedRuleDefault(
             name='foo:create_bar',
@@ -1588,8 +1675,6 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             description='Create a bar.',
             operations=[{'path': '/v1/bars', 'method': 'POST'}],
             deprecated_rule=deprecated_rule,
-            deprecated_reason='"role:bang" is a better default',
-            deprecated_since='N'
         )]
         self.enforcer.register_defaults(rule_list)
 
@@ -1624,7 +1709,9 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
         # Deprecate the policy name in favor of something better.
         deprecated_rule = policy.DeprecatedRule(
             name='old_rule',
-            check_str='role:bang'
+            check_str='role:bang',
+            deprecated_reason='"old_rule" is a bad name',
+            deprecated_since='N',
         )
         rule_list = [policy.DocumentedRuleDefault(
             name='new_rule',
@@ -1632,8 +1719,6 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             description='Replacement for old_rule.',
             operations=[{'path': '/v1/bars', 'method': 'POST'}],
             deprecated_rule=deprecated_rule,
-            deprecated_reason='"old_rule" is a bad name',
-            deprecated_since='N'
         )]
         self.enforcer.register_defaults(rule_list)
 
@@ -1653,7 +1738,9 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
                                group='oslo_policy')
         deprecated_rule = policy.DeprecatedRule(
             name='foo:create_bar',
-            check_str='role:fizz'
+            check_str='role:fizz',
+            deprecated_reason='"role:bang" is a better default',
+            deprecated_since='N',
         )
 
         rule_list = [policy.DocumentedRuleDefault(
@@ -1662,8 +1749,6 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             description='Create a bar.',
             operations=[{'path': '/v1/bars', 'method': 'POST'}],
             deprecated_rule=deprecated_rule,
-            deprecated_reason='"role:bang" is a better default',
-            deprecated_since='N'
         )]
         enforcer = policy.Enforcer(self.conf)
         enforcer.register_defaults(rule_list)
@@ -1681,6 +1766,49 @@ class DocumentedRuleDefaultDeprecationTestCase(base.PolicyBaseTestCase):
             enforcer.enforce('foo:create_bar', {}, {'roles': ['baz']})
         )
 
+    def test_deprecation_logic_is_only_performed_once_per_rule(self):
+        deprecated_rule = policy.DeprecatedRule(
+            name='foo:create_bar',
+            check_str='role:fizz'
+        )
+        rule = policy.DocumentedRuleDefault(
+            name='foo:create_bar',
+            check_str='role:bang',
+            description='Create a bar.',
+            operations=[{'path': '/v1/bars', 'method': 'POST'}],
+            deprecated_rule=deprecated_rule,
+            deprecated_reason='"role:bang" is a better default',
+            deprecated_since='N'
+        )
+        check = rule.check
+
+        enforcer = policy.Enforcer(self.conf)
+        enforcer.register_defaults([rule])
+
+        # Check that rule processing hasn't been done, yet
+        self.assertEqual({}, enforcer.rules)
+
+        # Load the rules
+        enforcer.load_rules()
+
+        # Loading the rules will store a version of the rule check string
+        # logically ORed with the check string of the deprecated value. Make
+        # sure this is happening but that the original rule check is unchanged
+        expected_check = policy.OrCheck([
+            _parser.parse_rule(cs) for cs in
+            [rule.check_str, deprecated_rule.check_str]
+        ])
+        self.assertIn('foo:create_bar', enforcer.rules)
+        self.assertEqual(
+            str(enforcer.rules['foo:create_bar']), str(expected_check))
+        self.assertEqual(check, rule.check)
+        # Hacky way to check whether _handle_deprecated_rule was called again.
+        # If a second call to load_rules doesn't overwrite our dummy rule then
+        # we know it didn't call the deprecated rule function again.
+        enforcer.rules['foo:create_bar'] = 'foo:bar'
+        enforcer.load_rules()
+        self.assertEqual('foo:bar', enforcer.rules['foo:create_bar'])
+
 
 class DocumentedRuleDefaultTestCase(base.PolicyBaseTestCase):
 
@@ -1765,6 +1893,18 @@ class DocumentedRuleDefaultTestCase(base.PolicyBaseTestCase):
                           operations=invalid_op)
 
 
+class DeprecatedRuleTestCase(base.PolicyBaseTestCase):
+
+    def test_should_include_deprecated_meta(self):
+        with mock.patch('warnings.warn') as mock_warn:
+            policy.DeprecatedRule(
+                name='foo:bar',
+                check_str='rule:baz'
+            )
+
+            mock_warn.assert_called_once()
+
+
 class EnforcerCheckRulesTest(base.PolicyBaseTestCase):
     def setUp(self):
         super(EnforcerCheckRulesTest, self).setUp()
@@ -1831,3 +1971,83 @@ class EnforcerCheckRulesTest(base.PolicyBaseTestCase):
         self.enforcer.load_rules(True)
 
         self.assertTrue(self.enforcer.check_rules())
+
+
+class PickPolicyFileTestCase(base.PolicyBaseTestCase):
+
+    def setUp(self):
+        super(PickPolicyFileTestCase, self).setUp()
+        self.data = {
+            'rule_admin': 'True',
+            'rule_admin2': 'is_admin:True'
+        }
+        self.tmpdir = self.useFixture(fixtures.TempDir())
+        original_search_dirs = cfg._search_dirs
+
+        def fake_search_dirs(dirs, name):
+            dirs.append(self.tmpdir.path)
+            return original_search_dirs(dirs, name)
+
+        mock_search_dir = self.useFixture(
+            fixtures.MockPatch('oslo_config.cfg._search_dirs')).mock
+        mock_search_dir.side_effect = fake_search_dirs
+
+        mock_cfg_location = self.useFixture(
+            fixtures.MockPatchObject(self.conf, 'get_location')).mock
+        mock_cfg_location.return_value = cfg.LocationInfo(
+            cfg.Locations.set_default, 'None')
+
+    def test_no_fallback_to_json_file(self):
+        tmpfilename = 'policy.yaml'
+        self.conf.set_override('policy_file', tmpfilename, group='oslo_policy')
+        jsonfile = os.path.join(self.tmpdir.path, 'policy.json')
+        with open(jsonfile, 'w') as fh:
+            jsonutils.dump(self.data, fh)
+
+        selected_policy_file = policy.pick_default_policy_file(
+            self.conf, fallback_to_json_file=False)
+        self.assertEqual(self.conf.oslo_policy.policy_file, tmpfilename)
+        self.assertEqual(selected_policy_file, tmpfilename)
+
+    def test_overridden_policy_file(self):
+        tmpfilename = 'nova-policy.yaml'
+        self.conf.set_override('policy_file', tmpfilename, group='oslo_policy')
+        selected_policy_file = policy.pick_default_policy_file(self.conf)
+        self.assertEqual(self.conf.oslo_policy.policy_file, tmpfilename)
+        self.assertEqual(selected_policy_file, tmpfilename)
+
+    def test_only_new_default_policy_file_exist(self):
+        self.conf.set_override('policy_file', 'policy.yaml',
+                               group='oslo_policy')
+        tmpfilename = os.path.join(self.tmpdir.path, 'policy.yaml')
+        with open(tmpfilename, 'w') as fh:
+            yaml.dump(self.data, fh)
+
+        selected_policy_file = policy.pick_default_policy_file(self.conf)
+        self.assertEqual(self.conf.oslo_policy.policy_file, 'policy.yaml')
+        self.assertEqual(selected_policy_file, 'policy.yaml')
+
+    def test_only_old_default_policy_file_exist(self):
+        self.conf.set_override('policy_file', 'policy.yaml',
+                               group='oslo_policy')
+        tmpfilename = os.path.join(self.tmpdir.path, 'policy.json')
+        with open(tmpfilename, 'w') as fh:
+            jsonutils.dump(self.data, fh)
+
+        selected_policy_file = policy.pick_default_policy_file(self.conf)
+        self.assertEqual(self.conf.oslo_policy.policy_file, 'policy.yaml')
+        self.assertEqual(selected_policy_file, 'policy.json')
+
+    def test_both_default_policy_file_exist(self):
+        self.conf.set_override('policy_file', 'policy.yaml',
+                               group='oslo_policy')
+        tmpfilename1 = os.path.join(self.tmpdir.path, 'policy.json')
+        with open(tmpfilename1, 'w') as fh:
+            jsonutils.dump(self.data, fh)
+        tmpfilename2 = os.path.join(self.tmpdir.path, 'policy.yaml')
+        with open(tmpfilename2, 'w') as fh:
+            yaml.dump(self.data, fh)
+
+        selected_policy_file = policy.pick_default_policy_file(self.conf)
+        self.assertEqual(self.conf.oslo_policy.policy_file, 'policy.yaml')
+        self.assertEqual(selected_policy_file, 'policy.yaml')
